@@ -14,6 +14,7 @@ Client class for callers of the Rucio system
 
 from rucio.client.accountclient import AccountClient
 from rucio.client.didclient import DIDClient
+from rucio.client.lockclient import LockClient
 from rucio.client.metaclient import MetaClient
 from rucio.client.pingclient import PingClient
 from rucio.client.replicaclient import ReplicaClient
@@ -23,7 +24,7 @@ from rucio.client.scopeclient import ScopeClient
 from rucio.client.subscriptionclient import SubscriptionClient
 
 
-class Client(AccountClient, MetaClient, PingClient, ReplicaClient, RSEClient, ScopeClient, DIDClient, RuleClient, SubscriptionClient):
+class Client(AccountClient, MetaClient, PingClient, ReplicaClient, RSEClient, ScopeClient, DIDClient, RuleClient, SubscriptionClient, LockClient):
 
     """Main client class for accessing Rucio resources. Handles the authentication."""
 
