@@ -20,10 +20,29 @@ Compatibility Wrapper for DQ2 and Rucio.
 import copy
 import hashlib
 import re
+import string
 
-from datetime import datetime
+from datetime import datetime, timedelta
 from rucio.client.client import Client
-from rucio.common.exception import DataIdentifierNotFound, Duplicate, InvalidMetadata, RSENotFound, NameTypeError, InputValidationError, UnsupportedOperation, ScopeNotFound, ReplicaNotFound, RuleNotFound, FileAlreadyExists
+from rucio.common.exception import DataIdentifierNotFound, Duplicate, InvalidMetadata, RSENotFound, NameTypeError, InputValidationError, UnsupportedOperation, ScopeNotFound, ReplicaNotFound, RuleNotFound, FileAlreadyExists, FileConsistencyMismatch
+
+
+def validate_time_formats(time_string):
+    try:
+        d = re.match(r'((?P<days>\d+) days?)?'
+                     r'(, )?'
+                     r'((?P<hours>\d+):(?P<minutes>\d+):(?P<seconds>\d+))?',
+                     str(time_string))
+
+        if not filter(lambda r: r[1], d.groupdict().items()):
+            err_msg = 'Parameter value [%s] is not a valid time delta !' % (time_string)
+            raise InputValidationError(err_msg)
+
+            delta = timedelta(**dict([(key, (value and int(value) or 0)) for key, value in d.groupdict(0).items()]))
+            return delta
+    except:
+        err_msg = 'Parameter value [%s] is not a valid time delta !' % (time_string)
+        raise InputValidationError(err_msg)
 
 
 def extract_scope(dsn):
@@ -158,7 +177,7 @@ class DQ2Client:
                 # if rule['account'] == account:
                 # print 'Will delete %s:%s from %s ruleid %s' % (scope, dsn, rule['rse_expression'], rule['id'])
                 self.client.delete_replication_rule(rule['id'])
-                result['rse_expression'] = {'status': True}
+                result[rule['rse_expression']] = {'status': True}
         for location in locations:
             if location not in result:
                 result[location] = {'status': False, 'error': RuleNotFound}
@@ -220,8 +239,9 @@ class DQ2Client:
             - DataIdentifierNotFound is raised in case the container or dataset name doesn't exist.
 
         @see: https://twiki.cern.ch/twiki/bin/view/Atlas/DonQuijote2ContainerCatalogUC0004
-        ToDo Cedric
         """
+        if name.endswith('/'):
+            name = name[:-1]
         raise NotImplementedError
 
     def deleteFilesFromDataset(self, dsn, guids=[], scope=None):
@@ -241,10 +261,12 @@ class DQ2Client:
         @return: List of lfns that failed to be added since they are duplicates?
         """
         dids = []
-        # create rucio parameter
-        for fn in guids:
-            did = {'scope': scope, 'name': fn}
-            dids.append(did)
+        for file in self.client.list_files(scope=scope, name=dsn, long=True):
+            guid = file['guid']
+            guid = '%s-%s-%s-%s-%s' % (guid[0:8], guid[8:12], guid[12:16], guid[16:20], guid[20:32])
+            if guid in guids or guid.upper() in guids:
+                did = {'scope': file['scope'], 'name': file['name']}
+                dids.append(did)
         self.client.detach_dids(scope=scope, name=dsn, dids=dids)
 
     def eraseDataset(self, dsn, scope):
@@ -274,7 +296,8 @@ class DQ2Client:
                 result[location]['error'] = 'Cannot delete replicating datasets'
                 will_erase = False
         if will_erase:
-            self.client.set_metadata(scope=scope, name=dsn, key='expired_at', value=datetime.now())
+            # self.client.set_metadata(scope=scope, name=dsn, key='expired_at', value=str(datetime.now()))
+            self.client.set_metadata(scope=scope, name=dsn, key='lifetime', value=0.0001)
         return result
 
     def freezeDataset(self, dsn, scope):
@@ -309,11 +332,25 @@ class DQ2Client:
             size += f['bytes']
         return size
 
-    def getMasterReplicaLocation(self):
+    def getMasterReplicaLocation(self, dsn, version=0, scope=None):
         """
-        ToDo --> N/A ?
+        Returns the master replicas location, in Rucio, this is the oldest rule.
+
+        @param dsn: is the dataset name.
+        @param version: the version (not used in Rucio.
+        @param scope: is the dataset scope.
+
+        B{Exceptions:}
+            - DataIdentifierNotFound is raised in case the dataset name doesn't exist.
         """
-        raise NotImplementedError
+        self.client.get_metadata(scope=scope, name=dsn)
+        oldest_rule = datetime.now()
+        rse = None
+        for rule in self.client.list_did_rules(scope=scope, name=dsn):
+            if rule['created_at'] < oldest_rule:
+                oldest_rule = rule['created_at']
+                rse = rule['rse_expression']
+        return rse
 
     def getMetaDataAttribute(self, dsn, attributes, version=0, scope=None):
         """
@@ -464,7 +501,7 @@ class DQ2Client:
         """
         result = {}
         files = []
-        dq2attrs = {'pdn': '', 'archived': 'primary', 'version': 0, 'checkstate': 6, 'transferState': 0, 'found': 0, 'total': 0, 'immutable': 0}
+        dq2attrs = {'pdn': '', 'archived': 'primary', 'version': 1, 'checkstate': 6, 'transferState': 0, 'found': 0, 'total': 0, 'immutable': 0}
         metadata = self.client.get_metadata(scope, name=dsn)
         # @immutable
         if not metadata['is_open']:
@@ -532,6 +569,8 @@ class DQ2Client:
           'state': row.state})
 
         """
+        if cn.endswith('/'):
+            cn = cn[:-1]
         result = {}
         replicas = {0: [], 1: []}
         self.client.get_metadata(scope=scope, name=cn)
@@ -660,7 +699,6 @@ class DQ2Client:
                         type = 'dataset'
                     elif metaDataAttributes[key] == 2:
                         type = 'container'
-            print scope, dsn, filters
             if long:
                 for name in self.client.list_dids(scope, filters, type):
                     meta = {'totalSize': 0, 'totalFiles': 0}
@@ -717,11 +755,51 @@ class DQ2Client:
                 raise InvalidMetadata
         raise NotImplementedError
 
-    def listDatasetsByNameInSite(self):
+    def listDatasetsByNameInSite(self, site, complete=None, name=None, p=None, rpp=None, group=None):
         """
-        ToDo
+        List datasets at site
+
+        @param site: is the location to be searched for.
+        @param complete: is the location state of the dataset at a site and may have
+            the following values: None: in which case the
+            location state is ignore; LocationState.COMPLETE: lists only datasets
+            fully present at the site (no files missing);
+            LocationState.INCOMPLETE: lists only datasets partially present at the
+            site (some files missing).
+        @param page: is the page to be displayed.
+        @param rpp: are the results per page.
+        @param group: Not used
+
+        B{Exceptions:}
+            - RSENotFound is raised in case the site doesn't exist.
+
+        @return: Tuple of dataset.
+            ('dsn1', 'dsn2'... )
         """
-        raise NotImplementedError
+        result = []
+        pattern = None
+        if name:
+            pattern = string.replace(name, '*', '.*')
+        for did in self.client.get_dataset_locks_by_rse(site):
+            scope = did['scope']
+            dsn = did['name']
+            state = did['state']
+            if pattern:
+                if re.match(pattern, dsn):
+                    match = True
+                else:
+                    match = False
+            else:
+                match = True
+            if complete == 1:
+                if state == 'OK' and match:
+                    result.append('%s:%s' % (scope, dsn))
+            elif complete == 0:
+                if state != 'OK' and match:
+                    result.append('%s:%s' % (scope, dsn))
+            elif match:
+                result.append('%s:%s' % (scope, dsn))
+        return tuple(result)
 
     def listDatasetsInContainer(self, cn, scope):
         """
@@ -733,6 +811,8 @@ class DQ2Client:
         {u'adler32': None, u'name': u'2013-12-30_12', u'bytes': None, u'scope': u'ams-2014-ISS.B700-pass5', u'type': u'DATASET', u'md5': None}, ....]
 
         """
+        if cn.endswith('/'):
+            cn = cn[:-1]
         ret = []
         try:
             if self.client.get_metadata(scope, cn)['did_type'] == 'CONTAINER':
@@ -754,11 +834,41 @@ class DQ2Client:
                     raise NameTypeError
         raise DataIdentifierNotFound
 
-    def listDatasetsInSite(self):
+    def listDatasetsInSite(self, site, complete=None, page=1, rpp=100):
         """
-        ToDo
+        List all the datasets and their versions available on
+        the given site.
+
+        @param site: is the location to be searched for.
+        @param complete: is the location state of the dataset at a site and may have
+            the following values: None: in which case the
+            location state is ignore; LocationState.COMPLETE: lists only datasets
+            fully present at the site (no files missing);
+            LocationState.INCOMPLETE: lists only datasets partially present at the
+            site (some files missing).
+        @param page: is the page to be displayed.
+        @param rpp: are the results per page.
+
+        B{Exceptions:}
+            - RSENotFound is raised in case the site doesn't exist.
+
+        @return: List of dataset versions.
+            {'dsn': [version_numberX,... version_numberY]}
         """
-        raise NotImplementedError
+        result = {}
+        for did in self.client.get_dataset_locks_by_rse(site):
+            scope = did['scope']
+            name = did['name']
+            state = did['state']
+            if complete == 1:
+                if state == 'OK':
+                    result['%s:%s' % (scope, name)] = [1]
+            elif complete == 0:
+                if state != 'OK':
+                    result['%s:%s' % (scope, name)] = [1]
+            else:
+                result['%s:%s' % (scope, name)] = [1]
+        return result
 
     def listFileReplicas(self, location, dsn, version=0, scope=None):
         """
@@ -990,17 +1100,37 @@ class DQ2Client:
             for rse in self.client.list_rses(rse_expression):
                 if rse not in list_rses:
                     list_rses.append(rse['rse'])
-        for replica in self.client.list_replicas([{'scope': scope, 'name': dsn}], unavailable=True):
-            for rse in replica['rses']:
-                if (rse in list_rses) and (rse not in result):
-                    result.append(rse)
+        result = list_rses
         return result
 
-    def listSubscriptionsInSite(self):
+    def listSubscriptionsInSite(self, site, long=False):
         """
-        ToDo
+        Returns a dict of all subscribed uids in a site containing all attributes.
+
+        @param site: is the dataset subscription dq2.location.
+        @param long: List dataset in long format (total sum for all the file sizes + total num of files).
+
+        B{Exceptions:}
+            - RSENotFound is raised in case the site doesn't exist.
+
+        @return: Returns a list of all subscribed uids in a site containing all attributes.
+            {'dsn': [versionX, versionY]}
         """
-        raise NotImplementedError
+        result = {}
+        for did in self.client.get_dataset_locks_by_rse(site):
+            scope = did['scope']
+            name = did['name']
+            state = did['state']
+            if long:
+                if state != 'OK':
+                    result['%s:%s' % (scope, name)] = {'totalSize': 0, 'version': 1, 'totalFiles': 0}
+                    for i in self.client.list_files(scope=scope, name=name):
+                        result['%s:%s' % (scope, name)]['totalFiles'] += 1
+                        result['%s:%s' % (scope, name)]['totalSize'] += i['bytes']
+            else:
+                if state != 'OK':
+                    result['%s:%s' % (scope, name)] = [1]
+        return result
 
     def listSuspiciousFiles(self):
         """
@@ -1095,6 +1225,8 @@ class DQ2Client:
         @rtype: NoneType
 
         """
+        if name.endswith('/'):
+            name = name[:-1]
         self.client.add_container(scope=scope, name=name)
         if datasets:
             # self.client.add_datasets_to_container(scope=scope, name=name, datasets=datasets)
@@ -1103,7 +1235,6 @@ class DQ2Client:
 
     def registerDatasetLocation(self, dsn, location, version=0, complete=0, group=None, archived=None, acl_alias=None, lifetime=None, pin_lifetime=None, scope=None):
         """
-        ToDo -->KuoHao
         Register new replica of a dataset(which must already defined in the repository)
 
         @param dsn: is the dataset name.
@@ -1123,12 +1254,25 @@ class DQ2Client:
             - DataIdentifierNotFound is raised in case the dataset name doesn't exist.
             - UnsupportedOperation is raised in case the location does not exist.
         """
+        if lifetime:
+            lifetime = validate_time_formats(lifetime)
+            if lifetime == timedelta(days=0, seconds=0, microseconds=0):
+                errMsg = 'lifetime must be greater than O!' % locals()
+                raise InputValidationError(errMsg)
+            lifetime = lifetime.days * 86400 + lifetime.seconds
+        if pin_lifetime:
+            pin_lifetime = validate_time_formats(pin_lifetime)
+            if pin_lifetime == timedelta(days=0, seconds=0, microseconds=0):
+                errMsg = 'pin_lifetime must be greater than O!' % locals()
+                raise InputValidationError(errMsg)
+            pin_lifetime = pin_lifetime.days * 86400 + pin_lifetime.seconds
+
         if location in [loc['rse'] for loc in self.client.list_rses()]:
             dids = []
             did = {'scope': scope, 'name': dsn}
             dids.append(did)
             # Add replication rule
-            self.client.add_replication_rule(dids=dids, copies=1, rse_expression=location, weight=None, lifetime=lifetime, grouping='DATASET', account=None, locked=False)
+            self.client.add_replication_rule(dids=dids, copies=1, rse_expression=location, weight=None, lifetime=lifetime, grouping='NONE', account=None, locked=False)
         else:
             raise UnsupportedOperation('Unknown RSE [%s]' % (location))
         return True
@@ -1164,6 +1308,19 @@ class DQ2Client:
         @param scope: is the dataset scope.
         """
         self.client.get_did(scope, dsn)
+        if replica_lifetime:
+            replica_lifetime = validate_time_formats(replica_lifetime)
+            if replica_lifetime == timedelta(days=0, seconds=0, microseconds=0):
+                errMsg = 'replica_lifetime must be greater than O!' % locals()
+                raise InputValidationError(errMsg)
+            replica_lifetime = replica_lifetime.days * 86400 + replica_lifetime.seconds
+
+        if pin_lifetime:
+            pin_lifetime = validate_time_formats(pin_lifetime)
+            if pin_lifetime == timedelta(days=0, seconds=0, microseconds=0):
+                errMsg = 'pin_lifetime must be greater than O!' % locals()
+                raise InputValidationError(errMsg)
+            pin_lifetime = pin_lifetime.days * 86400 + pin_lifetime.seconds
         if not owner:
             owner = 'root'
         else:
@@ -1202,6 +1359,8 @@ class DQ2Client:
         @see: https://twiki.cern.ch/twiki/bin/view/Atlas/DonQuijote2ContainerCatalogUC0011
 
         """
+        if name.endswith('/'):
+            name = name[:-1]
         dsns = []
         # create rucio parameter
         for ds in datasets:
@@ -1209,7 +1368,7 @@ class DQ2Client:
             dsns.append(dsn)
         self.client.add_datasets_to_container(scope=scope, name=name, dsns=dsns)
 
-    def registerFilesInDataset(self, dsn, lfns=[], guids=[], sizes=[], checksums=[], ignore=False, scope=None, rse=None):
+    def registerFilesInDataset(self, dsn, lfns=[], guids=[], sizes=[], checksums=[], ignore=False, scope=None, rse=None, pfns=[]):
         """
         ToDo-->KuoHao
 
@@ -1226,6 +1385,7 @@ class DQ2Client:
         @param rse: is the rse.
 
         """
+        result = {}
         # check dataset status (not closed)
         info = self.client.get_did(scope, dsn)
         if not (info['open']):
@@ -1233,28 +1393,49 @@ class DQ2Client:
 
         # merge lfn, guid, size, checksum into rucio file format
         files = []
+        index = 0
         for lfn, guid, size, checksum in zip(lfns, guids, sizes, checksums):
             if lfn.find(':') > -1:
                 s, lfn = lfn.split(':')[0], lfn.split(':')[1]
             else:
                 s = scope
-            file = {'scope': s, 'name': lfn, 'bytes': size, 'meta': {'guid': guid}}
+            try:
+                pfn = pfns[index]
+                file = {'scope': s, 'name': lfn, 'bytes': size, 'meta': {'guid': guid}, 'pfn': pfn}
+            except IndexError:
+                file = {'scope': s, 'name': lfn, 'bytes': size, 'meta': {'guid': guid}}
             if checksum.startswith('md5:'):
                 file['md5'] = checksum[4:]
             elif checksum.startswith('ad:'):
                 file['adler32'] = checksum[3:]
             files.append(file)
+            index += 1
         # add new file to dataset(rse need assign), in rucio rse is pre-assign(by user or group
-        self.client.add_files_to_dataset(scope=scope, name=dsn, files=files, rse=rse)
+        try:
+            self.client.add_files_to_dataset(scope=scope, name=dsn, files=files, rse=rse)
+            for lfn in lfns:
+                result[lfn] = {'status': True}
+        except (FileAlreadyExists, Duplicate, UnsupportedOperation):
+            for did in files:
+                try:
+                    self.client.add_files_to_dataset(scope=scope, name=dsn, files=[did], rse=rse)
+                    result[lfn] = {'status': True}
+                except (FileAlreadyExists, Duplicate):
+                    meta = self.client.get_metadata(did['scope'], did['name'])
+                    guid = meta['guid']
+                    guid = '%s-%s-%s-%s-%s' % (guid[0:8], guid[8:12], guid[12:16], guid[16:20], guid[20:32])
+                    if guid != did['meta']['guid']:
+                        result[lfn] = {'status': False, 'error': FileConsistencyMismatch('guid mismatch DDM %s vs user %s' % (guid, did['meta']['guid']))}
+                    elif meta['adler32'] != did['adler32']:
+                        result[lfn] = {'status': False, 'error': FileConsistencyMismatch('adler32 mismatch DDM %s vs user %s' % (meta['adler32'], did['adler32']))}
+                    elif meta['bytes'] != did['bytes']:
+                        result[lfn] = {'status': False, 'error': FileConsistencyMismatch('filesize mismatch DDM %s vs user %s' % (meta['bytes'], did['bytes']))}
+                except UnsupportedOperation, e:
+                    result[lfn] = {'status': False, 'error': e}
+        return result
 
-        # add existing file to dataset
-        # self.client.attach_dids(scope=scope, name=dsn, dids=files)
-        return True
-
-    def registerFilesInDatasets(self, datasets):
+    def registerFilesInDatasets(self, datasets, rse=None):
         """
-        ToDo-->KuoHao
-
         Add existing files to an existing dataset.(attach file to dataset)
 
         @param dataset: is a dictionary containing the dataset name and a list of its files.
@@ -1263,40 +1444,38 @@ class DQ2Client:
         """
         # Scope information need to be recorded in file information
         # ckeck dataset status (not closed)
-        for dsn in datasets:
-            scope, dsn = extract_scope(dsn)
+        for dataset in datasets:
+            scope, dsn = extract_scope(dataset)
             info = self.client.get_did(scope, dsn)
             if not (info['open']):
                 raise UnsupportedOperation
 
+        result = {}
         for dataset in datasets:
             scope, dsn = extract_scope(dataset)
+            vuid = hashlib.md5(scope+':'+dsn).hexdigest()
+            vuid = '%s-%s-%s-%s-%s' % (vuid[0:8], vuid[8:12], vuid[12:16], vuid[16:20], vuid[20:32])
+            result[vuid] = None
             # get file information
             lfns = []
             guids = []
             sizes = []
             checksums = []
-            scopes = []
+            pfns = []
             for file in datasets[dataset]:
-                lfns.append(file['lfn'])
                 guids.append(file['guid'])
                 sizes.append(file['size'])
                 checksums.append(file['checksum'])
-                scopes.append(file['scope'])
+                pfns.append(file['surl'])
+                if 'scope' not in file:
+                    if file['lfn'].find(':') > -1:
+                        lfns.append(file['lfn'])
+                    else:
+                        lfns.append('%s:%s' % (scope, file['lfn']))
+                else:
+                    lfns.append('%s:%s' % (file['scope'], file['lfn']))
 
-            # merge lfn, guid, size, checksum into rucio file format
-            files = []
-            for lfn, guid, size, checksum, scope in zip(lfns, guids, sizes, checksums, scopes):
-                file = {'scope': scope, 'name': lfn, 'bytes': size, 'meta': {'guid': guid}}
-                if checksum.startswith('md5:'):
-                    file['md5'] = checksum[4:]
-                elif checksum.startswith('ad:'):
-                    file['adler32'] = checksum[3:]
-                files.append(file)
-            # attach files to dataset
-            for scope in scopes:
-                self.client.attach_dids(scope=scope, name=dataset, dids=files)
-        return True
+            self.registerFilesInDataset(dsn, lfns=lfns, guids=guids, sizes=sizes, checksums=checksums, ignore=False, scope=scope, rse=rse, pfns=pfns)
 
     def registerNewDataset(self, dsn, lfns=[], guids=[], sizes=[], checksums=[], cooldown=None, provenance=None, group=None, hidden=False, scope=None, rse=None):
         """
@@ -1354,7 +1533,8 @@ class DQ2Client:
         vuid = hashlib.md5(scope+':'+dsn).hexdigest()
         vuid = '%s-%s-%s-%s-%s' % (vuid[0:8], vuid[8:12], vuid[12:16], vuid[16:20], vuid[20:32])
         duid = vuid
-        self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=rse, weight=None, lifetime=7*86400, grouping='DATASET', account=None, locked=False)
+        if rse:
+            self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=rse, weight=None, lifetime=None, grouping='DATASET', account=None, locked=False)
         return {'duid': duid, 'version': 1, 'vuid': vuid}
 
     def registerNewDataset2(self, dsn, lfns=[], guids=[], sizes=[], checksums=[], cooldown=None,  provenance=None, group=None, hidden=False, ignore=False, scope=None, rse=None):
@@ -1416,7 +1596,8 @@ class DQ2Client:
                         self.client.add_files_to_dataset(scope=scope, name=dsn, files=[f], rse=rse)
                     except FileAlreadyExists:
                         statuses[f['name']] = {'status': False, 'error': FileAlreadyExists, 'duid': duid}
-        self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=rse, weight=None, lifetime=7*86400, grouping='DATASET', account=None, locked=False)
+        if rse:
+            self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=rse, weight=None, lifetime=None, grouping='DATASET', account=None, locked=False)
         return {'duid': duid, 'version': 1, 'vuid': vuid}, statuses
 
     def registerNewVersion(self, dsn, lfns=[], guids=[], sizes=[], checksums=[], ignore=False, scope=None):
@@ -1545,17 +1726,67 @@ class DQ2Client:
         metadata_mapping = {'owner': 'account', 'lifetime': 'expired_at', 'hidden': 'hidden'}
         if attrname in metadata_mapping:
             self.client.set_metadata(scope=scope, name=dsn, key=attrname, value=attrvalue)
-        metadata_static = ['duid', 'state', 'creationdate', 'latestversion', 'lastoperationdn', 'lastoperationip', 'closeddate', 'frozendate', 'deleteddate', 'type',
-                           'freezingdate', 'group', 'provenance', 'nbfiles', 'length', 'vuid', 'version', 'versioncreationdate', 'latestvuid', 'origin', 'physicsgroup', 'temperature', '#replicas', 'tier0state', 'tier0type']
-        if attrname in metadata_static:
-            raise InvalidMetadata('%s is a static metadata and cannot be updated' % (attrname))
         raise InvalidMetadata('%s is not a valid DQ2 metadata' % (attrname))
 
     def setReplicaMetaDataAttribute(self, dsn, location, attrname, attrvalue, scope=None):
         """
-        ToDo Ookey
+        Set the value of the given attribute to the given
+        value for the given dataset replica. Operates on the current version.
+
+        @param dsn: is the dataset name.
+        @param location: is the location name.
+        @param attrname: is the metadata dataset attribute name.
+        @param attrvalue: is the metadata dataset attribute value.
+        @param scope: is dataset scope.
+
+        B{Exceptions:}
+            - DataIdentifierNotFound is raised in case the dataset name doesn't exist.
+            - InvalidMetadata is is raised in case of non valid attrname
+            - InputValidationError is case of non valid attrvalue
+            - UnsupportedOperation if the replica doesn't exist.
         """
-        raise NotImplementedError
+        attributes = ['group', 'owner', 'archived', 'pin_lifetime', 'lifetime']
+        if attrname not in attributes:
+            raise InvalidMetadata('%s is not a valid DQ2 replica metadata' % (attrname))
+        self.client.get_metadata(scope=scope, name=dsn)
+        is_at_site = False
+        account = self.client.account
+        if attrname == 'owner':
+            account = attrvalue
+        elif attrname == 'group':
+            account = attrvalue
+        lifetime = None
+        for rule in self.client.list_did_rules(scope=scope, name=dsn):
+            if rule['rse_expression'] == location:
+                is_at_site = True
+                if attrname == 'owner' or attrname == 'group':
+                    if rule['account'] == attrvalue:
+                        return 0
+                elif attrname == 'archived':
+                    # Does nothing
+                    pass
+                elif attrname == 'lifetime':
+                    lifetime = validate_time_formats(attrvalue)
+                    if lifetime == timedelta(days=0, seconds=0, microseconds=0):
+                        errMsg = 'lifetime must be greater than O!' % locals()
+                        raise InputValidationError(errMsg)
+                    lifetime = lifetime.days * 86400 + lifetime.seconds
+                    self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=rule['copies'], rse_expression=location, weight=rule['weight'], lifetime=lifetime, grouping=rule['grouping'], account=account, locked=False)
+                    self.client.delete_replication_rule(rule['id'])
+                elif attrname == 'pin_lifetime':
+                    if attrvalue is None or attrvalue is '':
+                        self.client.update_lock_state(rule['id'], lock_state=False)
+                    else:
+                        pin_lifetime = validate_time_formats(attrvalue)
+                        if pin_lifetime == timedelta(days=0, seconds=0, microseconds=0):
+                            errMsg = 'pin_lifetime must be greater than O!' % locals()
+                            raise InputValidationError(errMsg)
+                        pin_lifetime = pin_lifetime.days * 86400 + pin_lifetime.seconds
+                        self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=rule['copies'], rse_expression=location, weight=rule['weight'], lifetime=pin_lifetime, grouping=rule['grouping'], account=account, locked=True)
+        if not is_at_site:
+            raise UnsupportedOperation('Replicas for %s:%s at %s does not exist' % (scope, dsn, location))
+        self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=location, weight=None, lifetime=lifetime, grouping='DATASET', account=account, locked=False)
+        return 0
 
     def verifyFilesInDataset(self, dsn, guids, version=None, scope=None):
         """
