@@ -24,6 +24,22 @@ class ReplicaClient(BaseClient):
     def __init__(self, rucio_host=None, auth_host=None, account=None, ca_cert=None, auth_type=None, creds=None, timeout=None):
         super(ReplicaClient, self).__init__(rucio_host, auth_host, account, ca_cert, auth_type, creds, timeout)
 
+    def declare_bad_file_replicas(self, pfns, rse, session=None):
+        """
+        Declare a list of bad replicas.
+
+        :param pfns: The list of PFNs.
+        :param rse: The RSE name.
+        """
+        data = {'rse': rse, 'pfns': pfns}
+        url = build_url(self.host, path='/'.join([self.REPLICAS_BASEURL, 'badreplicas']))
+        headers = {}
+        r = self._send_request(url, headers=headers, type='POST', data=dumps(data))
+        if r.status_code == codes.created:
+            return True
+        exc_cls, exc_msg = self._get_exception(r.headers)
+        raise exc_cls(exc_msg)
+
     def get_did_from_pfns(self, pfns, rse):
         """
         Get the DIDs associated to a PFN on one given RSE
