@@ -6,7 +6,7 @@
 #
 # Authors:
 # - Vincent Garonne, <vincent.garonne@cern.ch>, 2012
-# - Martin Barisits, <martin.barisits@cern.ch>, 2013
+# - Martin Barisits, <martin.barisits@cern.ch>, 2013-2014
 # - Cedric Serfon, <cedric.serfon@cern.ch>, 2014
 
 from json import dumps, loads
@@ -26,25 +26,26 @@ class RuleClient(BaseClient):
     def __init__(self, rucio_host=None, auth_host=None, account=None, ca_cert=None, auth_type=None, creds=None, timeout=None):
         super(RuleClient, self).__init__(rucio_host, auth_host, account, ca_cert, auth_type, creds, timeout)
 
-    def add_replication_rule(self, dids, copies, rse_expression, weight=None, lifetime=None, grouping='DATASET', account=None, locked=False):
+    def add_replication_rule(self, dids, copies, rse_expression, weight=None, lifetime=None, grouping='DATASET', account=None, locked=False, source_replica_expression=None):
         """
-        :param dids:             The data identifier set.
-        :param copies:           The number of replicas.
-        :param rse_expression:   Boolean string expression to give the list of RSEs.
-        :param weight:           If the weighting option of the replication rule is used, the choice of RSEs takes their weight into account.
-        :param lifetime:         The lifetime of the replication rules (in hours).
-        :param grouping:         ALL -  All files will be replicated to the same RSE.
-                                 DATASET - All files in the same dataset will be replicated to the same RSE.
-                                 NONE - Files will be completely spread over all allowed RSEs without any grouping considerations at all.
-        :param account:          The account owning the rule.
-        :param locked:           If the rule is locked, it cannot be deleted.
+        :param dids:                       The data identifier set.
+        :param copies:                     The number of replicas.
+        :param rse_expression:             Boolean string expression to give the list of RSEs.
+        :param weight:                     If the weighting option of the replication rule is used, the choice of RSEs takes their weight into account.
+        :param lifetime:                   The lifetime of the replication rules (in hours).
+        :param grouping:                   ALL -  All files will be replicated to the same RSE.
+                                           DATASET - All files in the same dataset will be replicated to the same RSE.
+                                           NONE - Files will be completely spread over all allowed RSEs without any grouping considerations at all.
+        :param account:                    The account owning the rule.
+        :param locked:                     If the rule is locked, it cannot be deleted.
+        :param source_replica_expression:  RSE Expression for RSEs to be considered for source replicas.
         """
         path = self.RULE_BASEURL + '/'
         url = build_url(choice(self.list_hosts), path=path)
         # TODO remove the subscription_id from the client; It will only be used by the core;
         data = dumps({'dids': dids, 'copies': copies, 'rse_expression': rse_expression,
                       'weight': weight, 'lifetime': lifetime, 'grouping': grouping,
-                      'account': account, 'locked': locked})
+                      'account': account, 'locked': locked, 'source_replica_expression': source_replica_expression})
         r = self._send_request(url, type='POST', data=data)
         if r.status_code == codes.created:
             return loads(r.text)
