@@ -6,12 +6,13 @@
 # http://www.apache.org/licenses/LICENSE-2.0
 #
 # Authors:
-# - Vincent Garonne, <vincent.garonne@cern.ch>, 2013
+# - Vincent Garonne, <vincent.garonne@cern.ch>, 2013 - 2014
 # - Mario Lassnig, <mario.lassnig@cern.ch>, 2013
 # - Cedric Serfon, <cedric.serfon@cern.ch>, 2014
 
 import xmltodict
 
+from datetime import datetime
 from nose.tools import assert_equal, assert_in, assert_raises
 
 from rucio.db.constants import DIDType
@@ -19,15 +20,65 @@ from rucio.client.baseclient import BaseClient
 from rucio.client.didclient import DIDClient
 from rucio.client.replicaclient import ReplicaClient
 from rucio.common.config import config_get
-from rucio.common.exception import Duplicate, DataIdentifierNotFound
+from rucio.common.exception import DataIdentifierNotFound
 from rucio.common.utils import generate_uuid
 from rucio.core.did import add_did, attach_dids, get_did, set_status, list_files
-from rucio.core.replica import add_replica, add_replicas, delete_replicas, update_replica_lock_counter, get_replica, list_replicas
+from rucio.core.replica import add_replica, add_replicas, delete_replicas, update_replica_lock_counter,\
+    get_replica, list_replicas, declare_bad_file_replicas, list_bad_replicas, touch_replicas
 from rucio.rse import rsemanager as rsemgr
 
 
 class TestReplicaCore:
 
+    def test_add_list_bad_replicas(self):
+        """ REPLICA (CORE): Add bad replicas and list them"""
+        tmp_scope = 'mock'
+        nbfiles = 5
+        # Adding replicas to deterministic RSE
+        files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1L, 'adler32': '0cc737eb', 'meta': {'events': 10}} for i in xrange(nbfiles)]
+        rse_info = rsemgr.get_rse_info('MOCK')
+        rse_id1 = rse_info['id']
+        add_replicas(rse='MOCK', files=files, account='root')
+
+        # Listing replicas on deterministic RSE
+        replicas = []
+        list_rep = []
+        for replica in list_replicas(dids=[{'scope': f['scope'], 'name': f['name'], 'type': DIDType.FILE} for f in files], schemes=['srm']):
+            replicas.extend(replica['rses']['MOCK'])
+            list_rep.append(replica)
+        declare_bad_file_replicas(replicas, 'MOCK')
+        bad_replicas = list_bad_replicas()
+        nbbadrep = 0
+        for rep in list_rep:
+            for badrep in bad_replicas:
+                if badrep['rse_id'] == rse_id1:
+                    if badrep['scope'] == rep['scope'] and badrep['name'] == rep['name']:
+                        nbbadrep += 1
+        assert_equal(len(replicas), nbbadrep)
+
+        # Adding replicas to non-deterministic RSE
+        files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1L, 'adler32': '0cc737eb',
+                  'pfn': 'srm://mock2.com:8443/srm/managerv2?SFN=/rucio/tmpdisk/rucio_tests/%s/%s' % (tmp_scope, generate_uuid()), 'meta': {'events': 10}} for i in xrange(nbfiles)]
+        rse_info = rsemgr.get_rse_info('MOCK2')
+        rse_id2 = rse_info['id']
+        add_replicas(rse='MOCK2', files=files, account='root')
+
+        # Listing replicas on non-deterministic RSE
+        replicas = []
+        list_rep = []
+        for replica in list_replicas(dids=[{'scope': f['scope'], 'name': f['name'], 'type': DIDType.FILE} for f in files], schemes=['srm']):
+            replicas.extend(replica['rses']['MOCK2'])
+            list_rep.append(replica)
+        declare_bad_file_replicas(replicas, 'MOCK2')
+        bad_replicas = list_bad_replicas()
+        nbbadrep = 0
+        for rep in list_rep:
+            for badrep in bad_replicas:
+                if badrep['rse_id'] == rse_id2:
+                    if badrep['scope'] == rep['scope'] and badrep['name'] == rep['name']:
+                        nbbadrep += 1
+        assert_equal(len(replicas), nbbadrep)
+
     def test_add_list_replicas(self):
         """ REPLICA (CORE): Add and list file replicas """
         tmp_scope = 'mock'
@@ -105,12 +156,72 @@ class TestReplicaCore:
             assert_equal(replica['tombstone'] is None, tombstone)
             assert_equal(lock_counter, replica['lock_cnt'])
 
+    def test_touch_replicas(self):
+        """ REPLICA (CORE): Touch replicas accessed_at timestamp"""
+        tmp_scope = 'mock'
+        nbfiles = 5
+        files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1L, 'adler32': '0cc737eb', 'meta': {'events': 10}} for i in xrange(nbfiles)]
+        add_replicas(rse='MOCK', files=files, account='root')
+
+        touch_replicas(replicas=[{'scope': f['scope'], 'name': f['name'], 'rse': 'MOCK'} for f in files])
+        now = datetime.utcnow()
+        touch_replicas(replicas=[{'scope': f['scope'], 'name': f['name'], 'rse': 'MOCK', 'acessed_at': now} for f in files])
+
 
 class TestReplicaClients:
 
     def setup(self):
         self.replica_client = ReplicaClient()
 
+    def test_add_list_bad_replicas(self):
+        """ REPLICA (CLIENT): Add bad replicas"""
+        tmp_scope = 'mock'
+        nbfiles = 5
+        # Adding replicas to deterministic RSE
+        files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1L, 'adler32': '0cc737eb', 'meta': {'events': 10}} for i in xrange(nbfiles)]
+        rse_info = rsemgr.get_rse_info('MOCK')
+        rse_id1 = rse_info['id']
+        self.replica_client.add_replicas(rse='MOCK', files=files)
+
+        # Listing replicas on deterministic RSE
+        replicas = []
+        list_rep = []
+        for replica in self.replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm']):
+            replicas.extend(replica['rses']['MOCK'])
+            list_rep.append(replica)
+        self.replica_client.declare_bad_file_replicas(replicas, 'MOCK')
+        bad_replicas = list_bad_replicas()
+        nbbadrep = 0
+        for rep in list_rep:
+            for badrep in bad_replicas:
+                if badrep['rse_id'] == rse_id1:
+                    if badrep['scope'] == rep['scope'] and badrep['name'] == rep['name']:
+                        nbbadrep += 1
+        assert_equal(len(replicas), nbbadrep)
+
+        # Adding replicas to non-deterministic RSE
+        files = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1L, 'adler32': '0cc737eb',
+                  'pfn': 'srm://mock2.com:8443/srm/managerv2?SFN=/rucio/tmpdisk/rucio_tests/%s/%s' % (tmp_scope, generate_uuid()), 'meta': {'events': 10}} for i in xrange(nbfiles)]
+        rse_info = rsemgr.get_rse_info('MOCK2')
+        rse_id2 = rse_info['id']
+        self.replica_client.add_replicas(rse='MOCK2', files=files)
+
+        # Listing replicas on non-deterministic RSE
+        replicas = []
+        list_rep = []
+        for replica in self.replica_client.list_replicas(dids=[{'scope': f['scope'], 'name': f['name']} for f in files], schemes=['srm']):
+            replicas.extend(replica['rses']['MOCK2'])
+            list_rep.append(replica)
+        self.replica_client.declare_bad_file_replicas(replicas, 'MOCK2')
+        bad_replicas = list_bad_replicas()
+        nbbadrep = 0
+        for rep in list_rep:
+            for badrep in bad_replicas:
+                if badrep['rse_id'] == rse_id2:
+                    if badrep['scope'] == rep['scope'] and badrep['name'] == rep['name']:
+                        nbbadrep += 1
+        assert_equal(len(replicas), nbbadrep)
+
     def test_add_list_replicas(self):
         """ REPLICA (CLIENT): Add, change state and list file replicas """
         tmp_scope = 'mock'
@@ -119,9 +230,6 @@ class TestReplicaClients:
         files1 = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1L, 'adler32': '0cc737eb', 'meta': {'events': 10}} for i in xrange(nbfiles)]
         self.replica_client.add_replicas(rse='MOCK', files=files1)
 
-        with assert_raises(Duplicate):
-            self.replica_client.add_replicas(rse='MOCK', files=files1)
-
         files2 = [{'scope': tmp_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1L, 'adler32': '0cc737eb', 'meta': {'events': 10}} for i in xrange(nbfiles)]
         self.replica_client.add_replicas(rse='MOCK3', files=files2)
 
